import 'package:flutter/material.dart';
import 'package:vercoder_inputer/src/Options.dart';

typedef FinishInput = void Function(WGQVerCodeInputer inputer, String verCoder, BuildContext ctx);

abstract class InputerProtocol {
	void didFinishedInputer(WGQVerCodeInputer inputer, BuildContext ctx, String verCode);
}



class WGQVerCodeInputer extends StatefulWidget {
	int codeLength = 6;
	final Size size;
	double gap = 8.0;
	
	// final FinishInput finishInput;
	
	String get verCode {
		return myState.getVerCode();
	}
	
	InputerProtocol delegate;
	
	_InputerState myState;
	
	String getVerCode() {
		return myState.getVerCode();
	}
	
	void reset() {
		myState.reset();
	}
	
	Options options;
	
	WGQVerCodeInputer({
		this.codeLength,
		this.size,
		this.options,
		this.delegate,
	}) {
		if (options == null) {
			print('need init options');
			options = Options();
		}
		this.options = options;
	}
	
	@override
	_InputerState createState() {
		myState = _InputerState();
		return myState;
	}
}

class _InputerState extends State <WGQVerCodeInputer> {
	
	List <Widget> _inputers;
	List <TextField> _tfs;
	List <TextEditingController> tfCtrls;
	List <String> backStrs;
	List <FocusNode> fns;
	List <Key> keys;
	
	Options opt;
	
	@override
	initState() {
		super.initState();
		opt = widget.options;
		
		tfCtrls = List.generate(widget.codeLength, (_) {
			TextEditingController ctrl = TextEditingController();
//			ctrl.addListener(() {});
			return ctrl;
		});
		
		backStrs = List.generate(widget.codeLength, (_) {
			return "●";
		});
		
		fns = List.generate(widget.codeLength, (_) {
			FocusNode fn = FocusNode();
			return fn;
		});
		
		keys = List.generate(widget.codeLength, (_i){
			return Key(_i.toString());
		});
		
		WidgetsBinding.instance.addPostFrameCallback((_) {
			setState(() {
			
			});
		});
	}
	
	
	
	@override
	Widget build(BuildContext context) {
		_inputers = ttt();
		
		return SizedBox(
			height: widget.size.height,
			width: widget.size.width,
			child: ListView(
				padding: EdgeInsets.zero,
				children: _inputers,
				scrollDirection: Axis.horizontal,
			),
		);
	}
	
	void _handleChange(String text, int index) {
		String oldtxt = backStrs[index];
		print("第$index个字符改变,由$oldtxt变成$text");
		if (text.length == 0) {
			if (index > 0 && oldtxt == "●") {
				setFocusedTextFiled(index - 1);
				setTextFieldValue('●', index - 1);
				backStrs[index - 1] = '●';
				setState(() {});
			} else {
				setTextFieldValue('●', index);
				backStrs[index] = '●';
			}
		} else {
			setTextFieldValue(text, index);
			setFocusedTextFiled(index + 1);
			backStrs[index] = "";
		}
		
		///如果验证码全部输入完,调用回调
		if (checkCode()) {
			if (widget.delegate != null) {
				widget.delegate.didFinishedInputer(widget, context, getVerCode());
			}
		}
	}
	
	void focused(int index) {
		TextField tf = findTextFieldByIndex(index);
		TextEditingController ctrl = tfCtrls[index];
		
		if (tf.focusNode.hasFocus) {
			// print('$index 获得焦点');
			if (ctrl.text.isEmpty || ctrl.text == '●') {
				// print('$index 内容为空,设置为默认值');
				ctrl.text = '●';
			}
		}
		// print('$index 设置内容全选');
		setTextFieldSelectAll(index);
	}
	
	List<Widget> ttt() {
		_tfs = _buildTfs();
		return List.generate(widget.codeLength, (index) {
			FocusNode fc = fns[index];
			fc.addListener(() {
				focused(index);
			});
			
			///获取当前皮肤
			final ThemeData base = Theme.of(context);
			
			TextEditingController ctrl = tfCtrls[index];
//				Color borderColor = (ctrl.text.isEmpty || ctrl.text == '●') ? Colors.indigo : Colors.orange;
//			Color textColor = (ctrl.text.isEmpty || ctrl.text == '●') ? Colors.cyan : Colors.purple;
			Color borderColor = (ctrl.text.isEmpty || ctrl.text == '●') ? opt.emptyUnderLineColor : opt.inputedUnderLineColor;
			Color textColor = (ctrl.text.isEmpty || ctrl.text == '●') ? Colors.transparent : opt.fontColor;
			
			
			if (fc.hasFocus) {
				borderColor = opt.focusedColor;
			}

//			ThemeData buildDarkTheme() {
//				return base.copyWith(
//					textSelectionColor: Colors.blue,
//					secondaryHeaderColor: Colors.yellow,
//					accentColor: Colors.white,
//					inputDecorationTheme: InputDecorationTheme(
//						disabledBorder: new UnderlineInputBorder(
//							borderSide: new BorderSide(color: borderColor)),
//					),
//				);
//			}
			ThemeData buildDarkTheme() {
				return base.copyWith(
					textSelectionColor: Colors.transparent,
					secondaryHeaderColor: Colors.transparent,
					accentColor: Colors.white,
					inputDecorationTheme: InputDecorationTheme(
						disabledBorder: new UnderlineInputBorder(
							borderSide: new BorderSide(color: borderColor)),
					),
				);
			}
			
			TextField _tf = _tfs[index];

//			_tf.style = TextStyle(
//			fontSize: opt.fontSize,
//			color: textColor,
//			fontWeight: opt.fontWeight,
//			);
			
			return Container(
				width: (widget.size.width / widget.codeLength),
				
				child: Padding(
					padding: EdgeInsets.all(widget.gap),
					child: Theme(
						data: buildDarkTheme(),
						child: GestureDetector(
							onTap: () {
								setFocusedTextFiled(index);
							},
							behavior: HitTestBehavior.opaque,
							child: Container(
								child: _tf,
//								child: TextField(
//									controller: ctrl,
//									key: GlobalKey(),
//									textAlign: TextAlign.center,
//									keyboardType: TextInputType.number,
//									maxLength: 1,
//									cursorColor: Colors.transparent,
//									cursorWidth: 1,
//									style: TextStyle(
//										fontSize: opt.fontSize,
//										color: textColor,
//										fontWeight: opt.fontWeight,
//									),
//									decoration: InputDecoration(
//										counterText: "",
//									),
//									onChanged: (String text) {
//										_handleChange(text, index);
//									},
//									autofocus: index == 0,
//									focusNode: fns[index],
//								),
							),
						),
					),
				),
			);
		});
	}
	
	List<TextField> _buildTfs() {
		
		return List.generate(widget.codeLength, (_idx) {
			TextEditingController _ctl = tfCtrls[_idx];
//			bool dd = (_ctl.text.isEmpty || _ctl.text == '●');
			return TextField(
				controller: _ctl,
				key: keys[_idx],
//				key: Key(_idx.toString()),
				textAlign: TextAlign.center,
				keyboardType: TextInputType.number,
				maxLength: 1,
				cursorColor: Colors.transparent,
				cursorWidth: 1,
				style: TextStyle(
					fontSize: opt.fontSize,
					color: (_ctl.text.isEmpty || _ctl.text == '●') ? Colors.transparent : opt.fontColor,
//					color: opt.fontColor,
					fontWeight: opt.fontWeight,
				),
				decoration: InputDecoration(
					counterText: "",
				),
				onChanged: (String text) {
					_handleChange(text, _idx);
				},
				autofocus: _idx == 0,
				focusNode: fns[_idx],
			);
		});
	}
	
	bool checkCode() {
		bool flag = true;
		for (TextEditingController tf in tfCtrls) {
			if (tf.text.isEmpty || tf.text == "●") {
				flag = false;
				break;
			}
		}
		return flag;
	}
	
	void setTextFieldSelectAll(int tfIndex) {
		TextEditingController ctrl = tfCtrls[tfIndex];
		ctrl.selection = TextSelection(baseOffset: 0, extentOffset: ctrl.text.length);
	}
	
	void setFocusedTextFiled(int tfIndex) {
		if (tfIndex > widget.codeLength - 1 || tfIndex < 0) {
			return;
		}
//		TextField tf = findTextFieldByIndex(tfIndex);
		FocusNode fn = fns[tfIndex];

//		FocusScope.of(context).requestFocus(tf.focusNode);
		FocusScope.of(context).requestFocus(fn);
		setTextFieldSelectAll(tfIndex);
	}
	
	void setTextFieldValue(String text, int tfIndex) {
		TextEditingController ctrl = tfCtrls[tfIndex];
		ctrl.text = text;
		setTextFieldSelectAll(tfIndex);
		setState(() {});
	}
	
	TextField findTextFieldByIndex(int index) {
//		Container tfContainer = _inputers[index];
//		Padding padding = tfContainer.child;
//		Theme theme = padding.child;
//		GestureDetector gesture = theme.child;
//		Container container = gesture.child;
//		TextField tf = container.child;
		TextField tf = _tfs[index];
		return tf;
	}
	
	
	String getVerCode() {
		if (tfCtrls != null) {
			List <String> codeStr = tfCtrls.map((ctrl) {
				String char = ctrl.text == "●" ? "" : ctrl.text;
				return char;
			}).toList();
			return codeStr.join();
		}
		return "";
	}
	
	void reset() {
		for (TextEditingController ctrl in tfCtrls) {
			ctrl.text = "●";
		}
		FocusScope.of(context).requestFocus(fns[0]);
		setState(() {});
	}
}